/*
 * f_gauss.c
 *
 * V = f_gauss(x, y, z, ps, cs, sigma)
 *
 * Gauss Function f with correlation length sigma
 *
 * V[k*Nx*Ny + i*Ny + j] = f([x(i); y(j); z(k)])
 *
 */

#include <math.h>
#include <string.h>
#include <stdio.h>
#include "mex.h"

#define X_IN	prhs[0]
#define Y_IN    prhs[1]
#define Z_IN    prhs[2]
#define P_IN	prhs[3]
#define C_IN    prhs[4]
#define S_IN    prhs[5]
#define V_OUT	plhs[0]

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {
    
    double *x, *y, *z, *ps, *cs, *V, sigma;
    unsigned int Nx, Ny, Nz, Np;
    
    if(nrhs != 6 || nlhs != 1) {
        mexErrMsgTxt("f_gauss requires 6 input arguments and one output argument: \n \
                ==> V = f_gauss(x, y, z, ps, cs, sigma).");
    }
    
    /* read variables */
    x = mxGetPr(X_IN);
    y = mxGetPr(Y_IN);
    z = mxGetPr(Z_IN);
    ps = mxGetPr(P_IN);
    cs = mxGetPr(C_IN);
    sigma = mxGetScalar(S_IN);
    
    if(fabs(sigma) < 1e-15) {
        mexErrMsgTxt("sigma is almost zero, please specify an appropriate correlation length.");
    }
    
    Nx = mxGetN(X_IN);
    Ny = mxGetN(Y_IN);
    Nz = mxGetN(Z_IN);
    Np = mxGetN(P_IN);
    
    /* prepare output data */
    V_OUT = mxCreateDoubleMatrix(Nx*Ny*Nz, 1, mxREAL);
    
    /* define pointers to the out values */
    V = mxGetPr(V_OUT);
    memset(V, 0, Nx*Ny*Nz*sizeof(double));
    
    /* calculate the prefactors values */
    double prefac = 1.0/(sigma*sqrt(2*acos(-1.0))); /* 1 / (sqrt(2*pi)*sigma) */
    double fac = 1.0/(2*sigma*sigma); /* 1 / (2*sigma^2) */
    
    unsigned int i, j, k, l;
    
    for(k = 0; k < Nz; k++) {
        for(i = 0; i < Nx; i++) {
            for(j = 0; j < Ny; j++) {
                for(l = 0; l < Np; l++) {
                    /* TODO: implement */
                }
                V[k*Nx*Ny + i*Ny + j] *= prefac;
            }
        }
    }  
}
